function ExampleBICInterpolatorSimulation
global BoundaryNodeCoordinates BoundaryNodeLabels DesignUncoded MeshPoints tlist Data designsize nmesh ntlist
%Example in Appendix F
%checked 4
tlist = linspace(0,90,6)';
ntlist=length(tlist);
d=8;
%b1,b2,L,thick,hCoeff,ta,k,r
Max=[500 -100 0.2 0.0025 30 270 170 2.3*10^6];
Min=[600 0 0.3 0.005 60 300 230 2.5*10^6];
Range=Max-Min;

ntest=50;
xtest=lhsdesign(ntest,d,'criterion','maximin');
xtestuncoded=xtest.*repmat(Range,ntest,1)+repmat(Min,ntest,1);

[u, MeshPoints, e, t]=computetemperature(xtestuncoded(1,:));

N=length(u(:));
MeshPoints=MeshPoints';
nmesh=size(MeshPoints,1);

ytest=zeros(ntest,N);
for i=1:ntest
    u=computetemperature(xtestuncoded(i,:));
    ytest(i,:)=u(:)';
end

BoundaryNodeIndices=e(1:2,:);
BoundaryNodeIndices=BoundaryNodeIndices(:);
[BoundaryNodeIndices,uI]=unique(BoundaryNodeIndices);
BoundaryNodeCoordinates=MeshPoints(BoundaryNodeIndices,:);
BoundaryNodeLabels=[kron(e(5,:),ones(1,2))]';
BoundaryNodeLabels=BoundaryNodeLabels(uI);
grid=[repmat(MeshPoints,ntlist,1) kron(tlist,ones(nmesh,1))];

CorrelNo=1;
designsize=7*d;
nosim=1;
options=optimoptions(@fminunc,'MaxIter',10^3,'MaxFunEvals',10^5,'TolFun',10^-2,'TolX',10^-2,'Display','off','Algorithm','quasi-newton');

MSE=zeros(nosim,1);
MAE=zeros(nosim,1);
coverage=zeros(nosim,1);
avepredintlength=zeros(nosim,1);
time=zeros(nosim,1);

MSE2=zeros(nosim,1);
MAE2=zeros(nosim,1);
coverage2=zeros(nosim,1);
avepredintlength2=zeros(nosim,1);
time2=zeros(nosim,1);

MSE3=zeros(nosim,1);
MAE3=zeros(nosim,1);
coverage3=zeros(nosim,1);
avepredintlength3=zeros(nosim,1);
time3=zeros(nosim,1);

for run=1:nosim
Design=lhsdesign(designsize,d,'criterion','maximin');
DesignUncoded=Design.*repmat(Range,designsize,1)+repmat(Min,designsize,1);

Data=zeros(designsize,N);
for i=1:designsize
    [solution]=computetemperature(DesignUncoded(i,:));
    Data(i,:)=solution(:)';
end

tic
[thetaopt,beta,sigma2,R1I,R2I,RIRes]=FunctionalGPFit(Design,grid,Data(:),CorrelNo);
Ypred2=zeros(ntest,N);
UCL2=zeros(ntest,N); 
LCL2=zeros(ntest,N);
for i=1:ntest
[Ypred2(i,:), UCL2(i,:), LCL2(i,:)]=FunctionalGPPredict(Design,grid,Data(:),CorrelNo,[repmat(xtest(i,:),N,1) grid],thetaopt,beta,sigma2,R1I,R2I,RIRes);
end
toc
AbsErr2=abs(Ypred2-ytest);
MAE2(run)=mean(mean(AbsErr2));
MSE2(run)=mean(mean(AbsErr2.^2));
coverage2(run)=mean(mean((LCL2<=ytest).*(ytest<=UCL2)));
avepredintlength2(run)=mean(mean(UCL2-LCL2));
time2(run)=toc;

tic
[optlogpar, fval, exitflag] = fminunc(@OptimizeParameters,0,options)
opteta=exp(optlogpar);

Q=zeros(designsize,N);
for i=1:designsize
    BoundaryNodeValues=computeBoundaryNodeValues(DesignUncoded(i,1:2),BoundaryNodeCoordinates,BoundaryNodeLabels);
    InitialFunctionVal=u0func(MeshPoints,DesignUncoded(i,1:2));
    Yhat=BoundaryInitialConditionInterpolator(MeshPoints,tlist,opteta,BoundaryNodeCoordinates,tlist,BoundaryNodeValues,MeshPoints,InitialFunctionVal);
    Q(i,:)=Yhat';
end

transformeddata=Data-Q;

[EigVec,EigVal,zeta,MY]=functionalPCA(transformeddata);

J=find((cumsum(EigVal)/sum(EigVal))>0.999,1,'first');
EL=EigVec(:,1:J)*diag(sqrt(EigVal(1:J)));

zetapred=zeros(J,ntest);
zetaPosCov=cell(1,J);
for i=1:J
    [zetapred(i,:),zetaPosCov{i}]=GPFitPredict(Design,zeta(:,i),CorrelNo,xtest,2);
end
posmean=EL*zetapred;

poscov=cell(1,ntest);
for i=1:ntest
    C=zeros(J,J);
    for j=1:J
        C(j,j)=zetaPosCov{j}(i,i);
    end
    poscov{i}=EL*C*EL';
end
PredictTest=posmean;

BItest=zeros(N,ntest);
for i=1:ntest
BoundaryNodeValues=computeBoundaryNodeValues(xtestuncoded(i,1:2),BoundaryNodeCoordinates,BoundaryNodeLabels);
InitialFunctionVal=u0func(MeshPoints,xtestuncoded(i,1:2));
BItest(:,i)=BoundaryInitialConditionInterpolator(MeshPoints,tlist,opteta,BoundaryNodeCoordinates,tlist,BoundaryNodeValues,MeshPoints,InitialFunctionVal);
end
PredictTest=PredictTest+BItest+repmat(MY,1,ntest);

UCL=zeros(N,ntest);
LCL=zeros(N,ntest);
for i=1:ntest
    UCL(:,i)=PredictTest(:,i)+norminv(0.99)*sqrt(diag(poscov{i}));
    LCL(:,i)=PredictTest(:,i)-norminv(0.99)*sqrt(diag(poscov{i}));    
end
toc

AbsErr=abs(PredictTest'-ytest);
MAE(run)=mean(mean(AbsErr));
MSE(run)=mean(mean(AbsErr.^2));
coverage(run)=mean(mean((LCL'<=ytest).*(ytest<=UCL')));
avepredintlength(run)=mean(mean(UCL-LCL));
time(run)=toc;




tic

[EigVec,EigVal,zeta,MY]=functionalPCA(Data);

J=find((cumsum(EigVal)/sum(EigVal))>0.999,1,'first');
EL=EigVec(:,1:J)*diag(sqrt(EigVal(1:J)));

zetapred=zeros(J,ntest);
zetaPosCov=cell(1,J);
for i=1:J
    [zetapred(i,:),zetaPosCov{i}]=GPFitPredict(Design,zeta(:,i),CorrelNo,xtest,2);
end
posmean=EL*zetapred;

poscov=cell(1,ntest);
for i=1:ntest
    C=zeros(J,J);
    for j=1:J
        C(j,j)=zetaPosCov{j}(i,i);
    end
    poscov{i}=EL*C*EL';
end
PredictTest=posmean+repmat(MY,1,ntest);

UCL=zeros(N,ntest);
LCL=zeros(N,ntest);
for i=1:ntest
    UCL(:,i)=PredictTest(:,i)+norminv(0.99)*sqrt(diag(poscov{i}));
    LCL(:,i)=PredictTest(:,i)-norminv(0.99)*sqrt(diag(poscov{i}));    
end
toc

AbsErr=abs(PredictTest'-ytest);
MAE3(run)=mean(mean(AbsErr));
MSE3(run)=mean(mean(AbsErr.^2));
coverage3(run)=mean(mean((LCL'<=ytest).*(ytest<=UCL')));
avepredintlength3(run)=mean(mean(UCL-LCL));
time3(run)=toc;

end
xlswrite('simulationAppendixF.xlsx',MAE,'1','A1')
xlswrite('simulationAppendixF.xlsx',MSE,'1','B1')
xlswrite('simulationAppendixF.xlsx',coverage,'1','C1')
xlswrite('simulationAppendixF.xlsx',avepredintlength,'1','D1')
xlswrite('simulationAppendixF.xlsx',time,'1','E1')
xlswrite('simulationAppendixF.xlsx',MAE2,'2','A1')
xlswrite('simulationAppendixF.xlsx',MSE2,'2','B1')
xlswrite('simulationAppendixF.xlsx',coverage2,'2','C1')
xlswrite('simulationAppendixF.xlsx',avepredintlength2,'2','D1')
xlswrite('simulationAppendixF.xlsx',time2,'2','E1')
xlswrite('simulationAppendixF.xlsx',MAE3,'3','A1')
xlswrite('simulationAppendixF.xlsx',MSE3,'3','B1')
xlswrite('simulationAppendixF.xlsx',coverage3,'3','C1')
xlswrite('simulationAppendixF.xlsx',avepredintlength3,'3','D1')
xlswrite('simulationAppendixF.xlsx',time3,'3','E1')

function MSE=OptimizeParameters(par)
global BoundaryNodeCoordinates BoundaryNodeLabels DesignUncoded MeshPoints tlist Data designsize nmesh ntlist
%checked 4
eta=exp(par);
Prediction=zeros(designsize,nmesh*ntlist);

for i=1:designsize
    BoundaryNodeValues=computeBoundaryNodeValues(DesignUncoded(i,1:2),BoundaryNodeCoordinates,BoundaryNodeLabels);
    InitialFunctionVal=u0func(MeshPoints,DesignUncoded(i,1:2));
    Yhat=BoundaryInitialConditionInterpolator(MeshPoints,tlist,eta,BoundaryNodeCoordinates,tlist,BoundaryNodeValues,MeshPoints,InitialFunctionVal);
    Prediction(i,:)=Yhat';
end

MSE=mean(mean((Data-Prediction).^2));

function [u, MeshPoints2, e, t]=computetemperature(parameters)
global b1 b2 L thick hCoeff ta k r tlist
%checked 4
if(min(size(parameters))~=1)
    display('error')
    return
end
b1=parameters(1);
b2=parameters(2);
L=parameters(3);
thick=parameters(4);
hCoeff=parameters(5);
ta=parameters(6);
k=parameters(7);
r=parameters(8);

numberOfPDE = 1;
pdem = createpde(numberOfPDE);

R1 = [3,4,-1,1,1,-1,-1,-1,1,1]';
R2 = [3,4,-1/4,1/4,1/4,-1/4,-1/4,-1/4,1/4,1/4]';
geom = [R1,R2];
% Names for the two geometric objects
ns = (char('R1','R2'))';
% Set formula
sf = 'R1-R2';
g = decsg(geom,sf,ns);
% Convert the DECSG geometry into a geometry object
% on doing so it is appended to the PDEModel
geometryFromEdges(pdem,g);

% Create the triangular mesh on the square with approximately
% ten elements in each direction.
%hmax = .3; % element size
%msh=generateMesh(pdem,'Hmax', hmax);
%MeshPoints2=pdem.Mesh.Nodes;
%t=pdem.Mesh.Elements;
[MeshPoints2,e,t] = initmesh(g,'Hmax',0.5);

%pdeplot(MeshPoints2,e,t)
%input('press enter')

a2=2*L^2*hCoeff;
f2=2*L^2*(hCoeff*ta)+L^2*thick*0;
r2=L^2*r*thick;
c=k*thick;

u0=u0func(MeshPoints2',[b1,b2]);

u=parabolic(u0,tlist,@pdebound,MeshPoints2,e,t,c,a2,f2,r2);

%figure(1),pdegplot(g,'EdgeLabels','on') 
%figure(2),pdeplot(MeshPoints2,e,t); 
%figure(2),pdesurf(MeshPoints2,t,u(:,1))
%figure(3),pdesurf(MeshPoints2,t,u(:,2))
%figure(4),pdesurf(MeshPoints2,t,u(:,end))
%input('press enter')

function [qmatrix,gmatrix,hmatrix,rmatrix] = pdebound(p,e,u,time)
global b1 b2
%checked 4
ne = size(e,2); % number of edges
qmatrix = zeros(1,ne);
gmatrix = qmatrix;

hmatrix = ones(1,2*ne);
rmatrix = hmatrix;
for k = 1:ne
    x1=p(1,e(1,k));
    x2=p(1,e(2,k));
    y1=p(2,e(1,k));
    y2=p(2,e(2,k));
    infdist1=((max(abs(x1),abs(y1))-0.25)/(3/4));
    infdist2=((max(abs(x2),abs(y2))-0.25)/(3/4));
    if(ismember(e(5,k),[1 2 6 7])==1)
        rmatrix(k)=b1+b2*infdist1^2;
        rmatrix(k+ne)=b1+b2*infdist2^2;
    elseif(ismember(e(5,k),[3 4 5 8])==1)
        rmatrix(k)=b1+b2*infdist1^2;
        rmatrix(k+ne)=b1+b2*infdist2^2;
    end
end

function u0=u0func(MeshPoints,b)
%checked 4
np = size(MeshPoints,1); % Number of mesh points
u0 = zeros(np,1); % Allocate initial matrix
for k = 1:np
    x = MeshPoints(k,1);
    y = MeshPoints(k,2);
    infdist=((max(abs(x),abs(y))-0.25)/(3/4));
    u0(k) = (b(1)+b(2)*infdist^2); % Fill in row k
end

function BoundaryNodeVals=computeBoundaryNodeValues(x,BoundaryNodeCoordinates,BoundaryNodeLabels)
%checked 4
nb=length(BoundaryNodeLabels);
BoundaryNodeVals=zeros(nb,1);
for i=1:nb
    infdist=((max(abs(BoundaryNodeCoordinates(i,1)),abs(BoundaryNodeCoordinates(i,2)))-0.25)/(3/4));
    if(ismember(BoundaryNodeLabels(i),[1 2 6 7]))        
        BoundaryNodeVals(i)=x(1)+x(2)*infdist^2;
    else
        BoundaryNodeVals(i)=x(1)+x(2)*infdist^2;        
    end
end
