function [thetaopt,beta,sigma2,R1I,R2I,RIRes]=FunctionalGPFit(Din1,Din2,Yin,Noin)
%checked 4

global D1 D2 Y F n No n1 n2 d1 d2 Range

No=Noin;
D1=Din1;
D2=Din2;
Y=Yin;
[n1, d1]=size(D1);
[n2, d2]=size(D2);
n=n1*n2;
d=d1+d2;
F=ones(n,1);

%Range=[ones(1,d)]; %for plate heat transfer examples
Range=[ones(1,d1) max(D2)-min(D2)]; % for tissue heating example

options=optimoptions(@fminunc,'MaxIter',10^3,'TolX',0.005+0.0005*d,'TolFun',10^-5,'MaxFunEvals',10^5,'Display','off','Algorithm','quasi-newton');

if(No==0)
    [paropt1, fval1, exitflag1]=fminunc(@Obj,[norminv((0.25.^(1./Range.^2)).*ones(1,d))],options); 
    [paropt2, fval2, exitflag2]=fminunc(@Obj,[norminv((0.75.^(1./Range.^2)).*ones(1,d))],options);
elseif(No==1)
    [paropt1, fval1, exitflag1]=fminunc(@Obj,[log(0.91*ones(1,d).*Range)],options); 
    [paropt2, fval2, exitflag2]=fminunc(@Obj,[log(2.55*ones(1,d).*Range)],options);      
elseif(No==2)
    [paropt1, fval1, exitflag1]=fminunc(@Obj,log(0.89*ones(1,d).*Range),options); 
    [paropt2, fval2, exitflag2]=fminunc(@Obj,log(2.24*ones(1,d).*Range),options);    
end

if((exitflag1<=0)||(exitflag2<=0))
    display('Likelihood optimization failure: algorithm did not converge.')   
end
if(fval1<fval2)
    if(No==0)
        thetaopt=normcdf(paropt1);
    elseif(No==1)
        thetaopt=exp(paropt1);        
    elseif(No==2)
        thetaopt=exp(paropt1);        
    end
else
    if(No==0)
        thetaopt=normcdf(paropt2);
    elseif(No==1)
        thetaopt=exp(paropt2);  
    elseif(No==2)
        thetaopt=exp(paropt2);          
    end
end
display('Optimal values of GP parameters')
disp(thetaopt)
R1=zeros(n1,n1);
R2=zeros(n2,n2);
for i=1:n1
    for j=i:n1
        R1(i,j)=CompCorr(D1(i,:),D1(j,:),thetaopt(1:d1));
    end
end
for i=2:n1
    for j=1:(i-1)    
        R1(i,j)=R1(j,i);
    end
end
for i=1:n2
    for j=i:n2
        R2(i,j)=CompCorr(D2(i,:),D2(j,:),thetaopt((d1+1):(d1+d2)));
    end
end
for i=2:n2
    for j=1:(i-1)    
        R2(i,j)=R2(j,i); 
    end
end

R1I=invandlogdet(R1);
R2I=invandlogdet(R2);
RIF=R1I*ones(n1,n2)*R2I;
RIF=RIF(:);
beta=(RIF'*Y)/(F'*RIF);
Res=Y-F*beta;
RIRes=(R1I*reshape(Res,n1,n2)*R2I);
RIRes=RIRes(:);
sigma2=Res'*RIRes/n;

function Objective=Obj(par)
global D1 D2 Y F n No n1 n2 d1 d2 Range

if(No==0)
    r=normcdf(par);
%    if((sum(r<0.001)>0)||(sum(r>0.999)>0)) %for plate heat transfer examples
    if((sum(r<(0.001.^(1./Range.^2)))>0)||(sum(r>(0.999.^(1./Range.^2)))>0)) % for tissue heating example
        Objective=Inf;
        return
    end
elseif(No==1)
    r=exp(par);    
%    if((sum(r>16.4888)>0)) %for plate heat transfer examples
    if((sum(r>(56.4888.*Range))>0)) % for tissue heating example
        Objective=Inf;
        return        
    end      
elseif(No==2)
    r=exp(par);
%    if(sum(r>40.7953930912641)>0) %for plate heat transfer examples
    if(sum(r>(40.7953930912641.*Range))>0) % for tissue heating example
        Objective=Inf;
        return        
    end
end

R1=zeros(n1,n1);
R2=zeros(n2,n2);
for i=1:n1
    for j=i:n1
        R1(i,j)=CompCorr(D1(i,:),D1(j,:),r(1:d1));
    end
end
for i=2:n1
    for j=1:(i-1)    
        R1(i,j)=R1(j,i);
    end
end
for i=1:n2
    for j=i:n2
        R2(i,j)=CompCorr(D2(i,:),D2(j,:),r((d1+1):(d1+d2)));
    end
end
for i=2:n2
    for j=1:(i-1)    
        R2(i,j)=R2(j,i); 
    end
end

[R1I,LD1]=invandlogdet(R1);
[R2I,LD2]=invandlogdet(R2);
LDR=(n2)*LD1+(n1)*LD2;
RIF=(R1I*ones(n1,n2)*R2I);
RIF=RIF(:);
beta=(RIF'*Y)/(F'*RIF);
Res=Y-F*beta;
RIRes=R1I*reshape(Res,n1,n2)*R2I;
RIRes=RIRes(:);
sigma2=Res'*RIRes/(n);
Objective=n*log(max(sigma2,0))+LDR;

function Corr=CompCorr(x1,x2,r)
global No
if(No==0)
    Corr=prod(r.^((x1-x2).^2));
elseif(No==1)
    rho=x1-x2;
    rho1=sqrt(6)*abs(rho)./r;
    Corr=prod((exp(-rho1)).*(rho1+1));
elseif(No==2)
    rho=x1-x2;
    rho1=2*sqrt(2.5)*abs(rho)./r;
    Corr=prod((exp(-rho1)/3).*(rho1.^2+3*rho1+3)); 
end