function [Ypred,PosCov]=GPFitPredict(Din,Yin,Noin,x,Choicein)
%checked 4
clear D Y F n No Choice
global D Y F n No Choice

No=Noin;
D=Din;
Y=Yin;
[n, d]=size(D);
F=ones(n,1);
Choice=Choicein;

options=optimoptions(@fminunc,'MaxIter',10^3,'TolX',0.005+0.0005*d,'TolFun',10^-5,'MaxFunEvals',10^5,'Display','off','Algorithm','quasi-newton');

if(No==0)
    [paropt1, fval1, exitflag1]=fminunc(@Obj,[norminv(0.25*ones(1,d))],options); 
    [paropt2, fval2, exitflag2]=fminunc(@Obj,[norminv(0.75*ones(1,d))],options);
elseif(No==1)
    [paropt1, fval1, exitflag1]=fminunc(@Obj,[log(0.91*ones(1,d))],options); 
    [paropt2, fval2, exitflag2]=fminunc(@Obj,[log(2.55*ones(1,d))],options);      
elseif(No==2)
    [paropt1, fval1, exitflag1]=fminunc(@Obj,log(0.89*ones(1,d)),options); 
    [paropt2, fval2, exitflag2]=fminunc(@Obj,log(2.24*ones(1,d)),options);    
end

if((exitflag1<=0)||(exitflag2<=0))
    display('Likelihood optimization failure: algorithm did not converge.')   
end
if(fval1<fval2)
    if(No==0)
        thetaopt=normcdf(paropt1);
    elseif(No==1)
        thetaopt=exp(paropt1);        
    elseif(No==2)
        thetaopt=exp(paropt1);        
    end
else
    if(No==0)
        thetaopt=normcdf(paropt2);
    elseif(No==1)
        thetaopt=exp(paropt2);  
    elseif(No==2)
        thetaopt=exp(paropt2);          
    end
end
display('Optimal values of GP parameters')
disp(thetaopt)
R=zeros(n,n);
for i=1:n
    for j=i:n
        R(i,j)=CompCorr(D(i,:),D(j,:),thetaopt);
    end
end
for i=2:n
    for j=1:(i-1)    
        R(i,j)=R(j,i);
    end
end
RI=invandlogdet(R);
if(Choice==1)
    RIF=RI*F;
    beta=(RIF'*Y)/(F'*RIF);
    Res=Y-F*beta;
    RIRes=RI*Res;
    sigma2=Res'*RIRes/n;    
elseif(Choice==2)
    RIRes=RI*Y;
    sigma2=1;
elseif(Choice==3)
    RIF=RI*F;
    beta=(RIF'*Y)/(F'*RIF);   
    Res=Y-F*beta;
    RIRes=RI*Res;
    sigma2=1;       
end

m=size(x,1);
R2=zeros(m,n);
for i=1:m
    for j=1:n
        R2(i,j)=CompCorr(x(i,:),D(j,:),thetaopt);
    end
end
if((Choice==1)||(Choice==3))
    Ypred=beta+R2*RIRes;
elseif(Choice==2)
    Ypred=R2*RIRes;    
end

if(nargout>1)
R3=zeros(m,m);
for i=1:m
    for j=i:m
        R3(i,j)=CompCorr(x(i,:),x(j,:),thetaopt);
    end
end
for i=2:m
    for j=1:(i-1)
        R3(i,j)=R3(j,i);
    end
end    
PosCov=sigma2*(R3-R2*RI*R2');
end

function Objective=Obj(par)
global D Y F n No Choice

if((sum(isnan(par))>0)||(sum(isinf(par))>0))
    Objective=Inf;
    return
end
    
if(No==0)
    r=normcdf(par);
    if((sum(r<0.001)>0)||(sum(r>0.999)>0))
        Objective=Inf;
        return
    end
elseif(No==1)
    r=exp(par);    
%    if((sum(r>16.4888)>0)) %for plate heat transfer examples
    if((sum(r>56.4888)>0)) % for tissue heating example
        Objective=Inf;
        return        
    end      
elseif(No==2)
    r=exp(par);
    if(sum(r>40.7953930912641)>0)
        Objective=Inf;
        return        
    end
end

R=zeros(n,n);
for i=1:n
    for j=i:n
        R(i,j)=CompCorr(D(i,:),D(j,:),r);
    end
end
for i=2:n
    for j=1:(i-1)    
        R(i,j)=R(j,i);
    end
end
[RI, LDR]=invandlogdet(R);
if(Choice==1)
    RIF=RI*F;
    beta=(RIF'*Y)/(F'*RIF);
    Res=Y-F*beta;
    sigma2=Res'*(RI*Res)/(n);
    Objective=n*log(max(sigma2,0))+LDR;    
elseif(Choice==2)
    Objective=Y'*(RI*Y)+LDR;
elseif(Choice==3)
    RIF=RI*F;
    beta=(RIF'*Y)/(F'*RIF);
    Res=Y-F*beta;
    Objective=Res'*(RI*Res)+LDR;     
end

function Corr=CompCorr(x1,x2,r)
global No
if(No==0)
    Corr=prod(r.^((x1-x2).^2));
elseif(No==1)
    rho=x1-x2;
    rho1=sqrt(6)*abs(rho)./r;
    Corr=prod((exp(-rho1)).*(rho1+1));
elseif(No==2)
    rho=x1-x2;
    rho1=2*sqrt(2.5)*abs(rho)./r;
    Corr=prod((exp(-rho1)/3).*(rho1.^2+3*rho1+3)); 
end